#include "m33.h"
#include <math.h>

M33::M33(V3 *_rows) {

  rows[0] = _rows[0];
  rows[1] = _rows[1];
  rows[2] = _rows[2];

}

M33::M33() {

  rows[0] = V3(1.0f, 0.0f, 0.0f);
  rows[1] = V3(0.0f, 1.0f, 0.0f);
  rows[2] = V3(0.0f, 0.0f, 1.0f);

}


V3& M33::operator[](int i) {

  return rows[i];

}

ostream& operator<<(ostream &ostr, M33 &m) {

  ostr << m[0] << endl;
  ostr << m[1] << endl;
  ostr << m[2] << endl;
  return ostr;

}

V3 M33::operator*(V3 &v){

  V3 ret;
  ret[0] = rows[0]*v;
  ret[1] = rows[1]*v;
  ret[2] = rows[2]*v;

  return ret;
  
}

V3 M33::GetColumn(int i) {

  V3 ret(rows[0][i], rows[1][i], rows[2][i]);
  return ret;

}

void M33::SetColumn(int i, V3 &v) {

  rows[0][i] = v[0];
  rows[1][i] = v[1];
  rows[2][i] = v[2];

}


M33 M33::operator*(M33 &op1) {

  M33 ret;
  ret.SetColumn(0, (*this)*op1.GetColumn(0));
  ret.SetColumn(1, (*this)*op1.GetColumn(1));
  ret.SetColumn(2, (*this)*op1.GetColumn(2));
  return ret;

}

M33 M33::operator+(M33 &op1) {

  M33 ret;
  ret[0] = rows[0]+op1[0];
  ret[1] = rows[1]+op1[1];
  ret[2] = rows[2]+op1[2];
  return ret;

}

M33 M33::Inverted() {

  M33 ret;
  V3 a = GetColumn(0), b = GetColumn(1), c = GetColumn(2);
  V3 _a = b ^ c;
  V3 _b = c ^ a;
  V3 _c = a ^ b;
  _a = _a / (a * _a);
  _b = _b / (b * _b);
  _c = _c / (c * _c);
  ret[0] = _a;
  ret[1] = _b;
  ret[2] = _c;

  return ret;

}

M33 M33::Transposed() {

  M33 ret;
  ret.SetColumn(0, rows[0]);
  ret.SetColumn(1, rows[1]);
  ret.SetColumn(2, rows[2]);

  return ret;

}

M33::M33(char whichAxis, float theta) {

  switch(whichAxis) {
    case 'x': {
      cerr << "INFO: do not know how to do this" << endl;
      break;
    }
    case 'y': {
      float thetaRad = theta / 180.0f * 3.14159265f;
      float ct = cosf(thetaRad);
      float st = sinf(thetaRad);
      rows[0] = V3(ct, 0.0f, st);
      rows[1] = V3(0.0f, 1.0f, 0.0f);
      rows[2] = V3(-st, 0.0f, ct);
      break;
    }
    case 'z': {
      cerr << "INFO: do not know how to do this" << endl;
      break;
    }
    default: {
      cerr << "ERROR: do not know axis: " << whichAxis << endl;
    }
  }

}